from spotipyio.tools.extractors.entity_extractor_interface import IEntityExtractor
from spotipyio.tools.extractors.primary_artist_entity_extractor import PrimaryArtistEntityExtractor
from spotipyio.tools.extractors.track_entity_extractor import TrackEntityExtractor

__all__ = [
    "IEntityExtractor",
    "PrimaryArtistEntityExtractor",
    "TrackEntityExtractor",
]
