SPOTIFY_API_BASE_URL = "https://api.spotify.com/v1"
ID = "id"
IDS = "ids"
TRACK = "track"
TRACKS = "tracks"
ITEMS = "items"
TIME_RANGE = "time_range"
LIMIT = "limit"
NAME = "name"
DESCRIPTION = "description"
PUBLIC = "public"
URI = "uri"
URIS = "uris"
PLAYLISTS = "playlists"
IMAGES = "images"
ARTISTS = "artists"
POSITION = "position"
RANGE_START = "range_start"
INSERT_BEFORE = "insert_before"
SNAPSHOT_ID = "snapshot_id"
RANGE_LENGTH = "range_length"
NEXT = "next"
ADDITIONAL_TYPES = "additional_types"
USERS = "users"
OWNER = "owner"
HREF = "href"
OFFSET = "offset"
TOTAL = "total"
ALBUMS = "albums"
AUDIO_FEATURES_ROUTE = "audio-features"
AUDIO_FEATURES = "audio_features"
CHAPTERS = "chapters"
EPISODES = "episodes"
SHOWS = "shows"
AUDIOBOOKS = "audiobooks"
PREVIOUS = "previous"
