TOKEN_REQUEST_URL = "https://accounts.spotify.com/api/token"
ACCESS_CODE_REQUEST_URL = "https://accounts.spotify.com/authorize/"
REDIRECT_URI = "redirect_uri"
CODE = "code"
GRANT_TYPE = "grant_type"
JSON = "json"
REFRESH_TOKEN = "refresh_token"
CLIENT_ID = "client_id"
ACCESS_TOKEN = "access_token"
EXPIRES_IN = "expires_in"
SCOPE = "scope"
TOKEN_TYPE = "token_type"
RESPONSE_TYPE = "response_type"
